// This is a sweeper source code, support ad9833 and ad9850 as a DDS Chip
// use some arduino libraries, then combined into one .ino file , easy to use
// Version 2, USE one LCD 1602  and a rotary encoder
// Version 2.1 extend lowest freq from 20K to 100Hz
// Version 2.2 add Square wave frequency mark method and 
//             A setting menu include frequency mark way choice,
//             calibrate crystal frequency, set to default , save setting
// Version 2.3 add wave form selection: Sine Wave TRIANGLE Wave, SUQARE Wave
//             add Sweep Speed selection: 40Hz - 1Hz
// Version 2.4 adjust menu item
// Version 2.5 
// Version 2.6 optmize for sweep speed
// Version 2.7  optmize for program size
// Version 2.9  audio extend 20 - 20000Hz
#include <Arduino.h>
#include <SPI.h>
#include <LiquidCrystal.h>
#include <EEPROM.h>

// audioFreqList for audio sweep
// 0 - 20  [20]-40 [40]-80 [60]-160 [80] -320 [100]=630  [120]-1300  [140]-2500 [160]-5000 [180]-10000 [200]-20000

const PROGMEM uint16_t audioFreqList[]={20,20,22,22,24,24,25,26,26,27,28,29,30,31,33,34,35,36,37,39,40,41,43,44,46,48,49,
               51,53,55,57,59,61,63,65,67,70,72,75,77,80,83,86,89,92,95,98,100,100,110,110,120,120,
               120,130,130,140,140,150,150,160,160,170,180,180,190,200,200,210,220,220,230,240,250,
               260,270,280,290,300,310,320,330,340,350,360,380,390,400,420,430,450,460,480,500,510,
               530,550,570,590,610,630,660,680,700,730,750,780,810,830,860,890,920,960,990,1000,1100,
               1100,1100,1200,1200,1300,1300,1400,1400,1400,1500,1600,1600,1700,1700,1800,1800,1900,
               2000,2000,2100,2200,2300,2300,2400,2500,2600,2700,2800,2900,3000,3100,3200,3300,3400,
               3600,3700,3800,3900,4100,4200,4400,4500,4700,4800,5000,5200,5400,5600,5800,6000,6200,
               6400,6600,6800,7100,7300,7600,7900,8100,8400,8700,9000,9300,9700,10000,10300,10700,11100,
               11500,11900,12300,12700,13200,13600,14100,14600,15100,15700,16200,16800,17400,18000,18600,19300,20000};
#define AUDIO_DELAY 75  // 75*200 about 15s per period
               
#define DDS_USE_AD9850          //if you use AD9850 as DDS Chip,keep this line
#define DDS_USE_AD9833          //if you use AD9833 as DDS Chip, keep this line

#ifdef DDS_USE_AD9850
//---------------------------------------------------------
// define of  ad9850
class AD9850SPI //: virtual public vDDS
{
  public:
    AD9850SPI(int pinNumber);
    void Begin();
    void setfreq(double f);
    void setFactor(float factor);

  private:
    int FQ_UD;
    int RESET;
    uint32_t deltaphase;
    uint8_t phase;
    void update();
    void pulse(int pin);
    double calibFreq;
    double fDivCalibFreq;
};

AD9850SPI::AD9850SPI(int pinNumber) {
  FQ_UD = pinNumber;    //  pin to select AD9850 chip
  RESET = 8;    // pin to reset AD9850
  deltaphase = 0;
  phase = 0;
  calibFreq = 125000000.0;
  fDivCalibFreq = 4294967296.0 / calibFreq;
}

void AD9850SPI::Begin() {
  pinMode(FQ_UD, OUTPUT);
  pinMode(RESET, OUTPUT);
  pulse(RESET);
  pulse(FQ_UD);
  digitalWrite(FQ_UD, HIGH);
  update();  
}

void AD9850SPI :: setFactor(float factor) {
  if (factor == 1.0) return;
  calibFreq = 125000000.0*factor;
  fDivCalibFreq = 4294967296.0 / calibFreq;
}
void AD9850SPI::update() {
  digitalWrite(FQ_UD, LOW);
  SPI.setBitOrder(LSBFIRST);
  SPI.setDataMode(SPI_MODE0);
  for (int i = 0; i < 4; i++, deltaphase >>= 8) {
    SPI.transfer(deltaphase & 0xFF);
  }
  //SPI.transfer(phase & 0xFF);
  SPI.transfer(phase);
  digitalWrite(FQ_UD, HIGH);
}

void AD9850SPI::setfreq(double f) {
  deltaphase = f * fDivCalibFreq;
  phase = 0;
  update();
}

void AD9850SPI::pulse(int pin) {
  digitalWrite(pin, HIGH);
  digitalWrite(pin, LOW);
}
//-----------------------------------------------------
#endif     // end of ad9850 define

#ifdef DDS_USE_AD9833
/* -----------------------------------------------------------
 *   define of ad9833
 */
#define pow2_28       268435456L  // 2^28 used in frequency word calculation
#define BITS_PER_DEG    11.3777777777778  // 4096 / 360

#define RESET_CMD     0x0100    // Reset enabled (also CMD RESET)
#define SLEEP_MODE      0x00C0    // Both DAC and Internal Clock
#define DISABLE_DAC     0x0040
#define DISABLE_INT_CLK   0x0080
#define PHASE_WRITE_CMD   0xC000    // Setup for Phase write
#define PHASE1_WRITE_REG  0x2000    // Which phase register
#define FREQ0_WRITE_REG   0x4000    // 
#define FREQ1_WRITE_REG   0x8000
#define PHASE1_OUTPUT_REG 0x0400    // Output is based off REG0/REG1
#define FREQ1_OUTPUT_REG  0x0800    // ditto

typedef enum { SINE_WAVE = 0x2000, TRIANGLE_WAVE = 0x2002,
         SQUARE_WAVE = 0x2028, HALF_SQUARE_WAVE = 0x2020 } WaveformType;
typedef enum { REG0, REG1, SAME_AS_REG0 } Registers;

class AD9833 {
public:
    AD9833 ( uint8_t FNCpin);
  void Begin ( void );
  void ApplySignal ( WaveformType waveType, Registers freqReg,
    float frequencyInHz,
    Registers phaseReg = SAME_AS_REG0, float phaseInDeg = 0.0 );

  void Reset ( void );
  void setfreq(float frequency );
  void SetWaveform ( Registers waveFormReg, WaveformType waveType );
  void setWaveForm2(int form);
  void SetPhase ( Registers phaseReg, float phaseInDeg );
  void SetOutputSource ( Registers freqReg, Registers phaseReg );
  void EnableOutput ( bool enable );
  void setFactor(float factor);

private:
  void      WriteRegister ( int16_t dat );
  void      WriteRegister3 ( int16_t dat0, int16_t dat1 , int16_t dat2 );
  void      WriteControlRegister ( void );
  uint16_t    waveForm0; 
  uint16_t     registerValue;
  uint8_t     initWriteRegister;
  uint8_t     FNCpin;
  uint8_t     outputEnabled, DacDisabled, IntClkDisabled;
  float    refFrequency;
  float     frequency0, phase0;
  Registers   activeFreq, activePhase;
};

AD9833 :: AD9833 ( uint8_t pinNumber) {
  // Pin used to enable SPI communication (active LOW)
  this->FNCpin = pinNumber;
  pinMode(FNCpin,OUTPUT);
  digitalWrite(FNCpin,HIGH);
  refFrequency = 25e6; //25000000L;  // Chip working freq, AD9833 on module is 25MHz
  
  // Setup some defaults
  DacDisabled = false;
  IntClkDisabled = false;
  outputEnabled = false;
  waveForm0  = SINE_WAVE;
  frequency0  = 1000;   // 1 KHz sine wave to start
  phase0 =  0.0;        // 0 phase
  activeFreq = REG0; activePhase = REG0;
  initWriteRegister = 5;
}

void AD9833 :: Begin ( void ) {
  Reset();  // Hold in RESET until first WriteRegister command
  ApplySignal(SINE_WAVE,REG0,1000);
  EnableOutput(true);
}

void AD9833 :: setFactor(float factor) {
  if (factor == 1.0) return;
  refFrequency = 25e6 * factor;
}
void AD9833 :: ApplySignal ( WaveformType waveType,
    Registers freqReg, float frequencyInHz,
    Registers phaseReg, float phaseInDeg ) {
  setfreq ( frequencyInHz );
  SetPhase ( phaseReg, phaseInDeg );
  SetWaveform ( freqReg, waveType );
  SetOutputSource ( freqReg, phaseReg );
}

void AD9833 :: Reset ( void ) {
  WriteRegister(RESET_CMD);
  delay(15);
}

void AD9833 :: setfreq(float frequency ) {

  if ( frequency > 12.5e6 ) 
    frequency = 12.5e6;
  if ( frequency < 0.0 ) frequency = 0.0;
  frequency0 = frequency;

  int32_t freqWord = (frequency * pow2_28) / refFrequency;
  int16_t upper14 = (int16_t)((freqWord & 0xFFFC000) >> 14), 
      lower14 = (int16_t)(freqWord & 0x3FFF);

  lower14 |= FREQ0_WRITE_REG;  //reg;
  upper14 |= FREQ0_WRITE_REG;  //reg;   

  if (initWriteRegister)  { WriteControlRegister();
    WriteRegister(lower14);     // Write lower 14 bits to AD9833
    WriteRegister(upper14);     // Write upper 14 bits to AD9833
  } else {
    WriteRegister3(registerValue,lower14,upper14); // for sweep speed
  }  
}

void AD9833 :: SetPhase ( Registers phaseReg, float phaseInDeg ) {
  phaseInDeg = fmod(phaseInDeg,360);
  if ( phaseInDeg < 0 ) phaseInDeg += 360;
  uint16_t phaseVal = (uint16_t)(BITS_PER_DEG * phaseInDeg) & 0x0FFF;
  phaseVal |= PHASE_WRITE_CMD;
  phase0 = phaseInDeg;
  WriteRegister(phaseVal);
}

/*
 * Set the type of waveform that is output for a frequency register
 * SINE_WAVE, TRIANGLE_WAVE, SQUARE_WAVE, HALF_SQUARE_WAVE
 */
void AD9833 :: SetWaveform (  Registers waveFormReg, WaveformType waveType ) {
    waveForm0 = waveType;
  WriteControlRegister();
}

void AD9833 :: setWaveForm2 (int form) {
  initWriteRegister = 5;
  switch (form) {
    case 0:
      SetWaveform(REG0,SINE_WAVE);
      break;
    case 1:
      SetWaveform(REG0,TRIANGLE_WAVE);
      break;
    case 2:
      SetWaveform(REG0,SQUARE_WAVE);
      break;
    case 3:
      SetWaveform(REG0,HALF_SQUARE_WAVE);
      break;
  }
}

void AD9833 :: EnableOutput ( bool enable ) {
  outputEnabled = enable;
  WriteControlRegister();
}

void AD9833 :: SetOutputSource ( Registers freqReg, Registers phaseReg ) {
  activeFreq = freqReg;
  if ( phaseReg == SAME_AS_REG0 ) activePhase = activeFreq;
  else activePhase = phaseReg;
  WriteControlRegister();
}

// --------------------- PRIVATE FUNCTIONS --------------------------

void AD9833 :: WriteControlRegister ( void ) {
  uint16_t waveForm;
  if (initWriteRegister > 0) {
    waveForm = waveForm0;
    waveForm &= ~FREQ1_OUTPUT_REG;
    waveForm &= ~PHASE1_OUTPUT_REG;
  if ( outputEnabled )
    waveForm &= ~RESET_CMD;
  else
    waveForm |= RESET_CMD;
  if ( DacDisabled )
    waveForm |= DISABLE_DAC;
  else
    waveForm &= ~DISABLE_DAC;
  if ( IntClkDisabled )
    waveForm |= DISABLE_INT_CLK;
  else
    waveForm &= ~DISABLE_INT_CLK;
    registerValue = waveForm;
    initWriteRegister--;
  }
  WriteRegister ( registerValue );
}

void AD9833 :: WriteRegister ( int16_t dat ) {
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE2);
  digitalWrite(FNCpin,LOW);
  //delayMicroseconds(2); // Some delay may be needed
  SPI.transfer(highByte(dat));
  SPI.transfer(lowByte(dat));
  digitalWrite(FNCpin,HIGH);
}

void AD9833 :: WriteRegister3 ( int16_t dat0, int16_t dat1,int16_t dat2 ) {
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE2);
  digitalWrite(FNCpin,LOW);
  //delayMicroseconds(2); // Some delay may be needed
  SPI.transfer(highByte(dat0));
  SPI.transfer(lowByte(dat0));
  SPI.transfer(highByte(dat1));
  SPI.transfer(lowByte(dat1));
  SPI.transfer(highByte(dat2));
  SPI.transfer(lowByte(dat2));  
  digitalWrite(FNCpin,HIGH);
}
/*------------------------------------------------------------
 * end of define ad9833
 */
#endif

//------------------------------------------------------
// define of TLC5615
// we use PIN 3 for select chip
class TLC5615
{
  private:
    uint8_t _ss;
  public:
    TLC5615(uint8_t ss);
    void begin(void);
    void analogWrite(uint16_t value);
};

TLC5615::TLC5615(uint8_t ss)
{
  _ss = ss;
}

void TLC5615::begin(void)
{
  pinMode(_ss, OUTPUT);
  digitalWrite(_ss, 1);
}

void TLC5615::analogWrite(uint16_t value)
{
  value = (value & 0x3ff) << 2;
  digitalWrite(_ss, 0);
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE0);
  SPI.transfer16(value);
  digitalWrite(_ss, 1);
}
//------------------------------------------------------------

#define BUTTON_INTR_PIN 2
#define CHIP_SELECT_PIN 4
#define DAC_PIN 5
#define AD9833_PIN 6
#define AD9850_PIN 7

#define ENCODER_A_PIN 3
#define ENCODER_B_PIN 9
#define ENCODER_SWITCH_PIN 10
#define K1_SWITCH_PIN  2

#ifdef DDS_USE_AD9850
AD9850SPI DDS9850(AD9850_PIN); //use PIN 5 to select the chip to setfreq
#endif

#ifdef DDS_USE_AD9833
AD9833 DDS9833(AD9833_PIN);   //use PIN 4 to select the chip to setfreq
#endif
TLC5615 dac(DAC_PIN);  // use pin 3 for dac

//#define MAXMARKPOINT 5  // repeat times of every freq mark, the more repeat times ,the more bright
int MAXMARKPOINT = 5;
#define MAXCOUNT 200   // 200 points of each sweep period

struct settingData {
  char setName[4];
  char setVersion;
  char setSubVersion;
  uint16_t setLength;  // size include header
  char markMethod;  // 0 or 1
  float AD9833Factor;     // default is 1.0  , for calibrate frequency
  float AD9850Factor;
  char waveForm;  // 0,1,2   0-sin 1-Triangle 2-square
  char sweepSpeed; // 0-4, 0-40Hz 1-20Hz 2-10Hz 3-5Hz 4-1Hz
} settingData;

int  count = 0;
int markpoint = 0;
double startFreq, curFreq, endFreq, lastFreq;
double stepFreq,centralFreq;
double mode8StartF = 100e3;   // mode8 start 100k
double mode8EndF = 500e3;     // mode8 end 500k
double mode9StartF = 200e3;    // mode9 start 200K
int voltage = 0;        // value to set DAC output voltage
int mark1, mark2, mark3, mark4, mark5, mark6, mark7;
volatile int workingMode = 0; // 0-AM455K Sweep  1-455Konly  2-AM465K Sweep 3-465Konly  4-FM10.7M Sweep 5-10.7Monly
// 6-Audio Sweep   7-1000Hz only   8-Any scope Sweep  9-Any Frequency Generator
volatile int modeChanged = 1;
volatile int freqChanged = 0;
int chipInUse=1; // 1 = use AD9833  0=use AD9850
unsigned long changeModeMS[4];
int changeStartOrEnd=0;
int markMethod=0;   //0-bright freq mark, 1-square wave freq mark / use pin A0  to show 
char buf[18];
char myWaveForm=0; // onley for ad9833 0-SINE_WAVE, 1-TRIANGLE_WAVE, 2-SQUARE_WAVE, 3-HALF_SQUARE_WAVE
unsigned char sweepSpeed=1;  // sweepSpeed 0-40Hz 1-20Hz 2-10Hz 3-5Hz 4-1Hz 5-2s 6-5s 7-10s 8-20s
const unsigned int microDelay[]={0,125,125*3,125*7,125*39};  // 0-4 delayMicro
unsigned int delayCount;

LiquidCrystal lcd(A0,A1,A2,A3,A4,A5);

void setRotateProcess();
void setPushProcess();

void initSettingData() {
  settingData.setName[0]='S';
  settingData.setName[1]='E';
  settingData.setName[2]='T';
  settingData.setName[3]=':';
  settingData.setVersion=2;
  settingData.setSubVersion=2;
  settingData.setLength = sizeof(settingData); 
  settingData.markMethod=0;
  settingData.AD9833Factor=1.0;
  settingData.AD9850Factor=1.0;
  settingData.waveForm=0;  // sin form
  settingData.sweepSpeed=1; // 20Hz
}

void loadSettingData() {
  int i,len;
  byte *p;
  char sBuf[20];
  initSettingData();
  p = (byte *)&settingData;
  for (i=0;i<8;i++) {
    *p = EEPROM.read(i);
    p++;
  }
  if (!(settingData.setName[0] == 'S' && settingData.setName[1] == 'E' && settingData.setName[2] == 'T')) {
    Serial.println("No setting data");
    initSettingData();
    return;
  }

  len = settingData.setLength;
  settingData.setLength = sizeof(settingData);
  if (len > 100) {
    Serial.println("bat setting data");
    initSettingData();
    return;
  }
  if (len>sizeof(settingData)) len = sizeof(settingData);
  p = (byte *)&settingData;
  p = p+8;
  for(i=8;i<len;i++){
    *p = EEPROM.read(i);
    p++;
  }
  if (settingData.markMethod !=0 && settingData.markMethod != 1) {
    Serial.println("Bad markMethod");
  } else {
      markMethod=settingData.markMethod; 
      Serial.print("Init set markMethod to "); Serial.println(markMethod);
    }
    
  if (settingData.AD9850Factor > 1.2 or settingData.AD9850Factor <=0.8 or
      settingData.AD9833Factor > 1.2 or settingData.AD9833Factor <=0.8 ) {
    Serial.println("Bad factorCrystal");
  } else {
    DDS9850.setFactor(settingData.AD9850Factor);
    DDS9833.setFactor(settingData.AD9833Factor);
    Serial.print("set AD9850 factor to "); 
    dtostrf(settingData.AD9850Factor,1,8,sBuf);
    Serial.println(sBuf);
    Serial.print("set AD9833 factor to "); 
    dtostrf(settingData.AD9833Factor,1,8,sBuf);
    Serial.println(sBuf);
  }
  if (settingData.waveForm < 0 or settingData.waveForm >2) {
    Serial.println("Bad wave form value");    
  } else {
    myWaveForm=settingData.waveForm; 
    Serial.print("init set wave form to "); Serial.println(myWaveForm); 
  }
  if (settingData.sweepSpeed < 0 or settingData.sweepSpeed > 4) {
    Serial.println("Bad sweep speed");
  } else {
    sweepSpeed = settingData.sweepSpeed;
    Serial.print("init set sweep speed to "); Serial.println(sweepSpeed);
  }
}

void saveSettingData() {
  int i;
  byte *p;
  p = (byte *)&settingData;
  for(i=0;i<sizeof(settingData);i++) {
    EEPROM.write(i,*p);
    p++;
  }
}

void initEnv() {
  int pinValue,i;
  char *DDSname;
  lcd.begin(16,2);
  lcd.clear();
  lcd.print("DDS FR Sweep 2.9");
  pinMode(CHIP_SELECT_PIN,INPUT_PULLUP);
  pinValue = digitalRead(CHIP_SELECT_PIN);
  chipInUse = pinValue == LOW?0:1;
  DDSname = buf;
  if (pinValue == 0)
    strcpy(DDSname,"Using AD9850");
  else
    strcpy(DDSname,"Using AD9833");
  lcd.setCursor(0,1);
  lcd.print(DDSname);
  for (i=0;i<3;i++) changeModeMS[i]=0;
  SPI.begin();
  SPI.beginTransaction(SPISettings(14000000, LSBFIRST, SPI_MODE0));
  dac.begin();
  dac.analogWrite(0);
  DDS9850.Begin();
  DDS9833.Begin();
  loadSettingData();
  sweepSpeed=1;  // 20Hz
  delayCount=microDelay[sweepSpeed];
  delay(1000);
}

void showMode(int theWorkMode) {
  char *ms1,*ms2;
  char showBuf1[24],showBuf2[24];
  ms1 = showBuf1; ms2=showBuf2;
  switch (theWorkMode) {
    case 0:
      strcpy(ms1,"FR Sweep  455KHz");
      strcpy(ms2,"430K - 480KHz   ");
      break;
    case 1:
      strcpy(ms1,"Signal Generator");
      strcpy(ms2,"455KHz          ");
      break;
    case 2:
      strcpy(ms1,"FR Sweep  465KHz");
      strcpy(ms2,"440K - 490KHz   ");
      break;
    case 3:
      strcpy(ms1,"Signal Generator");
      strcpy(ms2,"465KHz          ");
      break;    
    case 4:      
      strcpy(ms1,"FR Sweep 10.7MHz");
      strcpy(ms2,"9.9M - 11.5MHz  ");
      break;      
    case 5:
      strcpy(ms1,"Signal Generator");
      strcpy(ms2,"10.7MHz         ");
      break;    
    case 6:
      strcpy(ms1,"FR Sweep Audio  ");
      strcpy(ms2,"20Hz - 20000Hz ");
      break;
    case 7:
      strcpy(ms1,"Signal Generator");
      strcpy(ms2,"1000Hz          ");
      break;
    case 8:
      //ms1="Any scope   200K";
      formatFreq(buf,centralFreq);
      strcpy(showBuf1,"Any scope ");
      strcat(showBuf1,buf);
      strcat(showBuf1,"   ");
      showBuf1[16]=0;
      ms1=showBuf1; 
      formatFreq(buf,startFreq);
      strcpy(showBuf2,buf);
      strcat(showBuf2," - ");
      formatFreq(buf,endFreq);
      strcat(showBuf2,buf);
      strcat(showBuf2,"      ");
      showBuf2[16]=0;
      ms2=showBuf2;
      break;
    case 9:
      strcpy(showBuf1,"Any freq        ");
      ms1 = showBuf1;
      formatFreq(showBuf2,startFreq);
      strcat(showBuf2,"             ");
      showBuf2[16]=0;
      ms2=showBuf2;
      break;
    case 10:
      showMenuItem(0);
      lcd.setCursor(0,0);
      lcd.print("Setting         ");
      return;
  }
  lcd.setCursor(0,0); lcd.print(ms1);
  lcd.setCursor(0,1); lcd.print(ms2);
  lcd.noCursor();
}
void initForFM() {
  startFreq = 9900000;   // Star Freq is 9.9MHz, end is 11.5, bandwidth is 1.6Mhz
  stepFreq = 8000;     // step is 8Khz
  endFreq = 11.5e6;
  mark1 = 63; // point is 10.4M
  mark2 = 75; // point is 10.5M
  mark3 = 88; // point is 10.6M
  mark4 = 100; // point is 10.7M
  mark5 = 113; // point is 10.8M
  mark6 = 125; // point is 10.9M
  mark7 = 138; // point is 11.0M
}

void initForAM455() {
  startFreq = 430000;   // Star Freq is 430KHz, end is 480KHz, bandwidth is 50KHz
  stepFreq = 250;     // step is 250hz
  endFreq=480000;
  mark1 = 40; // point is 440K
  mark2 = 60; // point is 445K
  mark3 = 80; // point is 450K
  mark4 = 100; // point is 455K
  mark5 = 120; // point is 460K
  mark6 = 140; // point is 465K
  mark7 = 160; // point is 470K
}

void initForAM465() {
  startFreq = 440000;   // Star Freq is 440KHz, end is 490KHz, bandwidth is 50KHz
  stepFreq = 250;     // step is 250hz
  endFreq=490000;
  mark1 = 40; // point is 450K
  mark2 = 60; // point is 455K
  mark3 = 80; // point is 460K
  mark4 = 100; // point is 465K
  mark5 = 120; // point is 470K
  mark6 = 140; // point is 475K
  mark7 = 160; // point is 480K
}

void initForAudio() {
  startFreq = pgm_read_word_near(audioFreqList+0);   // Star Freq defined by audioFreqList is 20Hz, end is 20000Hz, bandwidth is 20000Hz
  lastFreq=0;
  stepFreq = 100;     // step not use
  endFreq=20100;
  //  [40]-80 [60]-160 [80] -320 [100]=630  [120]-1300  [140]-2500 [160]-5000
  mark1 = 40; // point is 80
  mark2 = 60; // point is 160
  mark3 = 80; // point is 320
  mark4 = 100; // point is 630 central
  mark5 = 120; // point is 1300
  mark6 = 140; // point is 2500
  mark7 = 160; // point is 5000
}

void initForAnyScope() {
  startFreq = mode8StartF; //  100e3  start Freq is 100K
  endFreq = mode8EndF;    //   500e3   end Freq is 500K
  stepFreq = (endFreq-startFreq)/200.0;
  centralFreq = startFreq + stepFreq * 100;
  mark1 = 40; // point is 20%
  mark2 = 60; // point is 30%
  mark3 = 80; // point is 40%
  mark4 = 100; // point is 50%  central
  mark5 = 120; // point is 60%
  mark6 = 140; // point is 70%
  mark7 = 160; // point is 80%
}

void initForAnyFreq() {
  startFreq = mode9StartF;   // mode9 start 200kHz 
}

void initForInterrupt(){
  pinMode(BUTTON_INTR_PIN, INPUT_PULLUP);
  pinMode(ENCODER_A_PIN, INPUT_PULLUP);
  pinMode(ENCODER_B_PIN, INPUT_PULLUP);
  pinMode(ENCODER_SWITCH_PIN, INPUT_PULLUP);
    /* use PIN 2 (BUTTON_PIN and also Rotary Encode switch) for interrupt 0 */
  attachInterrupt(digitalPinToInterrupt(BUTTON_INTR_PIN),changeMode,FALLING);
  /* use PIN 3 connect to Rotary Encode for interrupt 1*/
  attachInterrupt(digitalPinToInterrupt(ENCODER_A_PIN), encoderIntr, CHANGE); 
}

volatile long lastIntrTime=0;
volatile int inBlink=0;
volatile int switchPressed=0;
volatile int encoderPushed=0;
volatile int encoderRotated=0;

void changeMode() {
  long theIntrTime;
  int whichKey,EncoderButton;
  if (inBlink) return;
  theIntrTime=millis();
  if ((theIntrTime - lastIntrTime) < 200){
    return;
  }
  switchPressed = 1;
  lastIntrTime = theIntrTime;  
  return;
}

void checkWhichPressed() {
  int whichKey,EncoderButton;
  whichKey = 0;
  SPI.end();
  pinMode(ENCODER_SWITCH_PIN,INPUT_PULLUP);
  delay(10);
  EncoderButton = digitalRead(ENCODER_SWITCH_PIN);
  SPI.begin();
  SPI.beginTransaction(SPISettings(14000000, LSBFIRST, SPI_MODE0));
  whichKey = EncoderButton == LOW ? 1:0;  // 0-K1 pressed, 1=Encoder button Pressed
  if (whichKey == 0) {
    workingMode++;
    if (workingMode > 10) workingMode = 0;
    modeChanged = 1;
  } else {
    encoderPushed=1;
  }
  switchPressed = 0;
}

/*
 * interrupt for encoder
 */

volatile long lastposition=0;
volatile long position=0;
volatile int positionChanged=0;
volatile int speed=1;
volatile int changedTimes=0;
volatile unsigned long lastEncoderMillis=0;
volatile float rotaryFreq=100e3;

void encoderIntr(){
  int changed;
  long theMillis;
  int direct=0;
  if (inBlink == 1) return;
  changed = 0;
  if (digitalRead(ENCODER_A_PIN) == LOW){   //  falling 
    // check channel B to watch if a CW event
    if (digitalRead(ENCODER_B_PIN) == LOW) {
      direct = 0;    // CW direction
      changed = 1;
    }
  }
  else{          // rising
    // check channel B to watch if a CCW  event
    if (digitalRead(ENCODER_B_PIN) == LOW) {
      direct = 1;
      changed = 1;
    }
  }
  if (changed) {
    theMillis = millis();
    if ((theMillis - lastEncoderMillis) > 2000) {
      changedTimes = 0;
      lastEncoderMillis = theMillis;
      if (speed > 1) {
        speed = 1;
      }
    } else {
      changedTimes++;
      if ((speed == 1 && changedTimes >=10) || (speed == 10 && changedTimes >= 15)) {
            speed = speed * 10;
          changedTimes = 0;
          lastEncoderMillis = theMillis;
      }
      if (speed == 100) {
        changedTimes = 0;
        lastEncoderMillis = theMillis;
      }
    }
    if (direct == 1) position = position - speed;
    else position = position + speed;
    encoderRotated = 1;
    positionChanged = (position - lastposition);
    Serial.print("New Positon:");Serial.println(position);
    lastposition = position;
  }
}

void formatFreq(char *strBuf,double f) {
  unsigned int KorMHz,fValue,i,digital,mv,j;
  // KorMHz  0 - unit Hz, 1 unit KHz,  2 unit MHz
  if (f < 10e3) {
    fValue = int(f + 0.5);
    KorMHz = 0;    // unit Hz
  }
  else if (f < 10e6) {
    fValue = int(f /1000.0 + 0.5);
    KorMHz = 1;  // unit KHz
  }
  else {
    fValue = int(f / 10000.0+0.5);
    KorMHz = 2; //unit MHz
  }
  mv = 1000;
  j=0;
  for (i=0;i<4;i++) {
    if (i==2) {
      if (KorMHz == 2) strBuf[j++] = '.';
    }
    digital = fValue/mv;
    if (j > 0) strBuf[j++]=digital+'0';
    else if (digital > 0)
      strBuf[j++]=digital+'0';
    fValue = fValue % mv;
    mv = mv  /10;
  }
  if (KorMHz > 0)
    strBuf[j++]=KorMHz==2?'M':'K';
  strBuf[j]=0;
 }

void processEncoderPushed() {
    encoderPushed = 0;
  if (workingMode == 10) {
    settingProcess(2);
    return;
  }
  changeStartOrEnd = !changeStartOrEnd;
}

void changeFreq1(double *f,int posi,double ref) {
  double ref1;      // modify the startFreq, 
  if (*f >= 10e6) {
    *f = *f + posi * 10000.0;
  } else if (*f >= 10e3) {
    *f = *f + posi * 1000.0;
  } else if (*f > 100.0) { 
    *f = *f + posi * 100.0; 
  } else { *f = *f + posi * 10.0; }
  if (*f < 20.0) *f = 20.0;     // lowest is 20 100(20k)
  if (ref >= 10e6) ref1 = ref - 10e3;
  else if (ref >= 10e3) ref1 = ref - 1e3;
  else ref1 = ref - 100.0;
  if (*f > ref1) *f = ref1;
  if (*f >= 10e6) {
    *f = (int(*f / 1e4+0.2)) * 1e4;
  } else if (*f >= 10e3) 
    *f = (int(*f / 1e3+0.2)) * 1e3;
}

void changeFreq2(double *f,int posi,double ref) {
  double ref1,maxref;  // modify the endFreq
  if (*f >= 10e6) {
    *f = *f + posi * 10000.0;
  } else if (*f >= 10e3) {
    *f = *f + posi * 1000.0;
  } else { *f = *f + posi * 100.0; }
  maxref=(chipInUse == 0)?70e6:12e6;   // use AD9855 max freq is 70M  AD9833 max freq is 12M
  if (*f < 100.0 ) *f = 100.0; 
  if (ref >= 10e6) ref1 = ref + 10e3;
  else if (ref >= 10e3) ref1 = ref + 1e3;
  else ref1 = ref + 100.0;
  if (*f < ref1) *f = ref1;  
  if (*f > maxref) *f = maxref;
  if (*f >= 10e6) {
    *f = (int(*f / 1e4+0.2)) * 1e4;
  } else if (*f >= 10e3) 
    *f = (int(*f / 1e3+0.2)) * 1e3;
}

void processEncoderRotated() {
  encoderRotated = 0;
  if (!(workingMode == 8 || workingMode == 9 || workingMode == 10)) return;
  if (positionChanged == 0) return; 
  if (workingMode == 8) {
    if (changeStartOrEnd == 0)
      changeFreq1(&mode8StartF,positionChanged,mode8EndF);
    else changeFreq2(&mode8EndF,positionChanged,mode8StartF);
    initForAnyScope();
    showMode(workingMode);
  }
  else if (workingMode == 9) {
    double maxref;
    maxref=(chipInUse == 0)?70e6:12e6;
    changeFreq1(&mode9StartF,positionChanged,maxref);
    initForAnyFreq();
    showMode(workingMode);
    freqChanged = 1;
  } else if (workingMode == 10) {
    settingProcess(1);
  }
}

char menuItem;  //0 - Frequency mark method 1-calibrate 2-Save 3-Rest
char cursorPosition;  // freqMark 11   cal: 4   Save: 6 Reset: 7
//int acceptRotate;
char bufIndex;
char inputLen;
char inputBuf[10];
char minInput[10];
char maxInput[10];
char keyAccept[10];
const char *tips[][5]={{"-Lght","-Sqr "},{"-40H","-20H","-10H","-5Hz","-1Hz"},{"-Sin","-Tri","-Sqr"}};


// menu 0: freqMark Method:  inputLen = 3  keyAccept: 0:y 1:n 2:y  range: 0:0-1 2:0-1
// menu 1: Sweep Speed
// menu 2: wave form
// menu 3: calibrate:2MHz  ad9833            inputlen = 9  keyAccpet: 0-6:y 7:n 8:y range 0:1-2 1-6:0-9 8:0-1
//         calibrate:10MHz ad9850           inputlen = 10 keyAccpet: 0-7:y 8:n 9:y range 0:0-1 1-7:0-9 9:0-1
// menu 4:default setting  inputlen = 1  keyAccpet: 0:y range 0:0-1
// menu 5:save             inputlen = 1  keyAccpet: 0:y range 0:0-1

void settingProcess(int event) {
  // event: 0  into setting menu
  //        1  rotate
  //        2  push button
  int i;
  switch (event) {
    case 0:
      menuItem = 0;     // "Mark: 0-Lght No "
      cursorPosition = 6;
      inputLen = 8;
      for (i=1;i<inputLen-1;i++) keyAccept[i]=0;
      keyAccept[0]=1;keyAccept[7]=1;
      minInput[0] = 0; maxInput[0]=1; minInput[7]=0; maxInput[7]=1;
      inputBuf[0]=markMethod; inputBuf[7]=1;
      bufIndex=0;
      showMenuItem(0);
      lcd.setCursor(cursorPosition,1);
      lcd.cursor();
      break;
    case 1:
      setRotateProcess();
      break;
    case 2:
      setPushProcess();
      break;      
  }
}

void showTips(){
  switch (menuItem) {
    case 0:  // freq mark method
      lcd.print(tips[0][inputBuf[0]]);
      break;
    case 2:  // wave form
      lcd.print(tips[2][inputBuf[0]]);
      break;
    case 1:  // sweep speed
      lcd.print(tips[1][inputBuf[0]]);
      break;
    default:
      break;
  }
}

void setRotateProcess() {
  char ch;
  ch = inputBuf[bufIndex];
  ch = ch + (positionChanged > 0 ? 1: -1);
  if (ch < minInput[bufIndex] || ch > maxInput[bufIndex]) return;
  inputBuf[bufIndex]=ch;
  lcd.setCursor(cursorPosition,1);
  if (inputLen == bufIndex+1) { // yes no
    if (ch == 0) lcd.print("Yes");
    else lcd.print("No ");
  } else {
    ch = ch + '0';
    lcd.print(ch);
    showTips();
  }
  lcd.setCursor(cursorPosition,1);
  return;  
}

void showMenuItem(int item) {
  int i;
  lcd.setCursor(0,1);
  switch (item) {
    case 0:
      lcd.print("Mark: ");
      lcd.print((char)('0'+inputBuf[0]));
      showTips();
      lcd.print(" No");
      lcd.setCursor(6,1);
      break;
    case 1:
      lcd.print("Speed: ");
      lcd.print((char)('0'+inputBuf[0]));
      lcd.print(tips[1][inputBuf[0]]);
      lcd.print(" No "); 
      break;
    case 2:
      lcd.print("Form: ");
      lcd.print((char)('0'+inputBuf[0]));
      lcd.print(tips[2][inputBuf[0]]);
      lcd.print(" No  ");
      lcd.setCursor(6,1);
      break;
    case 3:
      lcd.print("Cal:            ");
      lcd.setCursor(4,1);
      for (i=0;i<inputLen;i++) {
        if (keyAccept[i]==1) {
          if (i == inputLen-1) {
            if (inputBuf[i]==0) lcd.print("Yes");
            else lcd.print("No");
          } else 
            lcd.print((char)('0'+inputBuf[i]));       
        } 
        else lcd.print(' ');
      }
      break;
  }
  lcd.cursor();
}

void calibrateAD() {
  float realFreq,myFactor;
  int i;
  char showbuf[20];
  realFreq=0.0;
  for(i=0;i<inputLen-2;i++) {
    realFreq=realFreq*10 + inputBuf[i];
  }
  Serial.print("realfreq"); Serial.println(realFreq);
  if (chipInUse == 0) {
    myFactor = realFreq / 10e6;
    if (myFactor > 1.2 || myFactor < 0.8) {
      lcd.setCursor(0,1);
      lcd.print("Bad value");
      delay(500);
      return;
    }
    settingData.AD9850Factor = myFactor * settingData.AD9850Factor;
    
    DDS9850.setFactor(settingData.AD9850Factor );
    DDS9850.setfreq(10e6);
  } else {
    myFactor = realFreq / 2e6;
    if (myFactor > 1.2 || myFactor < 0.8) {
      lcd.setCursor(0,1);
      lcd.print("Bad value");
      delay(500);
      return;
    }
    settingData.AD9833Factor = myFactor * settingData.AD9833Factor;
    DDS9833.setFactor(settingData.AD9833Factor);
    DDS9833.setfreq(2e6);
  }
  myFactor = chipInUse == 0?settingData.AD9850Factor:settingData.AD9833Factor;
  Serial.print("Total Crystal factor: "); dtostrf(myFactor,12,10,showbuf); Serial.print(showbuf); 
}

void initSettingItem(int item){
  int i;
  bufIndex=0;
  switch (item) {
    case 0:
      break;
    case 3:
      lcd.setCursor(0,1);
      cursorPosition = 4;
      //9850                             9833
      // lcd.print("Cal:10000000 No "); lcd.print("Cal:2000000 No ");
      inputLen = chipInUse == 0 ? 10:9;
      for (i=0;i<inputLen;i++) keyAccept[i]=1;
      keyAccept[inputLen-2]=0;
      for (i=1;i<inputLen-2;i++){ minInput[i]=0; maxInput[i]=9; inputBuf[i]=0; }
      minInput[0]= chipInUse == 0 ? 0:1;
      maxInput[0]= chipInUse == 0 ? 1:2;
      minInput[inputLen-1]=0; maxInput[inputLen-1]=1;
      inputBuf[0] = chipInUse == 0 ? 1:2;   // init value
      inputBuf[inputLen-1]=1;
      showMenuItem(3);
      if (chipInUse == 0) {
        DDS9850.setfreq(10e6);
      } else DDS9833.setfreq(2e6);
      break;
    case 2:
      inputLen = 7;
      for (i=1;i<inputLen-1;i++) keyAccept[i]=0;
      keyAccept[0]=1;keyAccept[6]=1;
      minInput[0] = 0; maxInput[0]=2; minInput[6]=0; maxInput[6]=1;
      inputBuf[0]=myWaveForm; inputBuf[6]=1;
      showMenuItem(2);
      cursorPosition = 6;
      break;
    case 1:
      inputLen = 7;
      for (i=1;i<inputLen-1;i++) keyAccept[i]=0;
      keyAccept[0]=1;keyAccept[1]=0;keyAccept[6]=1;
      minInput[0] = 0; maxInput[0]=4; minInput[6]=0; maxInput[6]=1;
      inputBuf[0]=sweepSpeed; inputBuf[6]=1;
      lcd.cursor();
      lcd.setCursor(0,1);
      showMenuItem(1);
      cursorPosition = 7;  // "Speed: 0-40H No "  
      break;
    case 4:
      inputLen = 1;
      keyAccept[0]=1;
      minInput[0]=0; maxInput[0]=1;
      inputBuf[0] = 1;   // init value
      lcd.setCursor(0,1);
      lcd.print("Default: No        ");
      cursorPosition = 9;
      break;
    case 5:
      inputLen = 1;
      keyAccept[0]=1;
      minInput[0]=0; maxInput[0]=1;
      inputBuf[0] = 1;   // init value
      lcd.setCursor(0,1);
      lcd.print("Save: No        ");
      cursorPosition = 6;
      break;
  }
  lcd.cursor();
  lcd.setCursor(cursorPosition,1); 
}

void setPushProcess() {
  int i;

  if (menuItem == 0) {
    if (bufIndex < inputLen-1) {
      bufIndex++;
      cursorPosition++;
      for (i=1;i<inputLen;i++) {
        if (keyAccept[i] == 1) break;
        bufIndex++;
        cursorPosition++;
      }
      lcd.setCursor(cursorPosition,1);
      lcd.cursor();
    } else {
      lcd.setCursor(0,1);
      if (inputBuf[bufIndex] == 0) {
        //yes
        markMethod=inputBuf[0];
        settingData.markMethod = markMethod;
        lcd.print("Freq mark set...");
      } else {
        lcd.print("No change       ");
      }
        delay(500);
        menuItem++;
        initSettingItem(menuItem);     
    }
  }  // end of menuItem 0
  else if (menuItem == 3) {   // cal
    if (bufIndex < inputLen-1) {
      bufIndex++;
      cursorPosition++;
      if (keyAccept[bufIndex] == 0) {
        bufIndex++;
        cursorPosition++;        
      }
      lcd.cursor();
      lcd.setCursor(cursorPosition,1);       
    }
    else {
      lcd.setCursor(0,1);
      if (inputBuf[bufIndex] == 0) {
        //yes
        lcd.print("Calibrating ... ");
        calibrateAD();      
      } else 
        lcd.print("No change       ");
      delay(500);
      menuItem++;
      initSettingItem(menuItem);
    }
  }  // end of menuItem 1
  else if (menuItem == 2) {
     if (bufIndex < inputLen-1) {
      bufIndex++;
      cursorPosition++;
      for (i=1;i<inputLen;i++) {
        if (keyAccept[i] == 1) break;
        bufIndex++;
        cursorPosition++;
      }
      lcd.setCursor(cursorPosition,1);
      lcd.cursor();
    } else {
      lcd.setCursor(0,1);
      if (inputBuf[bufIndex] == 0) {
        //yes
        myWaveForm=inputBuf[0];
        settingData.waveForm = myWaveForm;
        DDS9833.setWaveForm2(myWaveForm);
        lcd.print("Set Form,Press M");
      } else {
        lcd.print("No act,  Press M");
      }
        delay(500);
            //to a new menuItem, set speed ,"Speed: ?-40H No "
        menuItem++;
        initSettingItem(menuItem);
    }    
  }
  else if (menuItem==1) {  // speed
    if (bufIndex < inputLen-1) {
      bufIndex++;
      cursorPosition++;
      for (i=1;i<inputLen;i++) {
        if (keyAccept[i] == 1) break;
        bufIndex++;
        cursorPosition++;
      }
      lcd.setCursor(cursorPosition,1);
      lcd.cursor();
    } else {
      lcd.setCursor(0,1);
      if (inputBuf[bufIndex] == 0) {
        //yes
        sweepSpeed=inputBuf[0];
        delayCount=microDelay[sweepSpeed];
        settingData.sweepSpeed = sweepSpeed;
        strcpy(buf,"Speed ?, Press M");
        buf[6]='0'+sweepSpeed;
        lcd.print(buf);
      } else {
        lcd.print("No act,  Press M");
      }
      delay(500);
        //to set default menu  
      menuItem++;
      if (chipInUse == 0) menuItem++; //if use AD9850 skip wave form
      initSettingItem(menuItem);
    }
  }
  else if (menuItem == 4) {
    lcd.setCursor(0,1);
    if (inputBuf[0] == 0) {  //yes
      lcd.print("To default...   ");  
      initSettingData();
      markMethod = 0;
      DDS9850.setFactor(1.0);
      DDS9833.setFactor(1.0);
      if (chipInUse == 0) {
          DDS9850.setfreq(10e6);
        } else DDS9833.setfreq(2e6);     
    } else { //no
      lcd.print("No change       ");
    }
    delay(500);
    //to a new menuItem  to save menu
    menuItem++;
    initSettingItem(menuItem);
  } 
  else if (menuItem == 5) {
    lcd.setCursor(0,1);
    if (inputBuf[0] == 0) {  //yes
      lcd.print("Save setting... "); 
      saveSettingData();
      delay(500);
      lcd.setCursor(0,1);
      lcd.print("OK, Press Mode  ");
    } else { //no
      lcd.print("No save, Press M");
    }
    delay(500); 
  }  
}

void blinkLed(int blinktimes) {
  int delayMS,i;
  inBlink = 1;
  delayMS = 100;
  SPI.end();
  pinMode(13,OUTPUT);
  digitalWrite(13,LOW);
  delay(300);
  for (i=0;i<blinktimes;i++) {
    digitalWrite(13,HIGH);
    delay(delayMS);
    digitalWrite(13,LOW);
    delay(delayMS);
  }
  SPI.begin();
  SPI.beginTransaction(SPISettings(14000000, LSBFIRST, SPI_MODE0));
  inBlink = 0;
}

void shiftModeMS() {
  changeModeMS[3]=changeModeMS[2];
  changeModeMS[2]=changeModeMS[1];
  changeModeMS[1]=changeModeMS[0];
  return;
}
/*This is the entry of arduino program
 * 
 */
void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  initEnv();
  initForInterrupt();
  markpoint = 0;
  count=0;
  workingMode=0;
  modeChanged=1;
}

void loop() {
  // put your main code here, to run repeatedly:
  unsigned long theMillis;
  if (switchPressed == 1) {
    inBlink = 1;
    Serial.println("swpress");
    checkWhichPressed();
    inBlink = 0;
  }
  if (encoderPushed == 1) {
    inBlink = 1;
    Serial.println("encoder pushed");
    processEncoderPushed();
    inBlink = 0;
  }
  if (encoderRotated == 1) {
    inBlink = 1;
    Serial.println("encoder rotated");
    processEncoderRotated();
    inBlink = 0;
  }
  if (modeChanged !=0 ) {
    inBlink = 1;
    theMillis = millis();
    if (((theMillis - changeModeMS[3]) < 1000) && (changeModeMS[3] > 0)) {
      //switch DDS chip
      Serial.print("Chip changed"); 
      chipInUse = chipInUse == 0?1:0;
      blinkLed(5);
      delay(1000);
      blinkLed(chipInUse==0?3:1);      
    }
    else 
      blinkLed(chipInUse==0?3:1); 
    shiftModeMS();
    changeModeMS[0] = theMillis;
    modeChanged = 0;
    count = 0;
    markpoint = 0;
    Serial.println(workingMode);
    Serial.print("Chip:"); Serial.println(chipInUse);
    switch (workingMode) {
      case 0:
          initForAM455();
          break;
      case 1:
          if (chipInUse == 0) {
            DDS9850.setfreq(455000);
          } else DDS9833.setfreq(455000);
          dac.analogWrite(0);
          break;
      case 2:
          initForAM465();
          break;
      case 3:
          if (chipInUse == 0) {
            DDS9850.setfreq(465000);
          } else DDS9833.setfreq(465000);
          dac.analogWrite(0);          
          break;
      case 4:
          initForFM();
          break;
      case 5:
          if (chipInUse == 0) {
            DDS9850.setfreq(10700000);
          } else DDS9833.setfreq(10700000);
          dac.analogWrite(0);
          break;
      case 6:
          initForAudio();
          break;
      case 7:
          if (chipInUse == 0) {
            DDS9850.setfreq(1000);
          } else DDS9833.setfreq(1000);
          dac.analogWrite(0);
          break;
      case 8:
          initForAnyScope();
          position = 0;
          lastposition = 0;
          break;
      case 9:
          initForAnyFreq();
          if (chipInUse == 0) {
            DDS9850.setfreq(startFreq);
          } else DDS9833.setfreq(startFreq);
          dac.analogWrite(0);   
          position = 0;
          lastposition = 0;    
          break; 
      case 10:  // for setting frequency mark method
          settingProcess(0);
          position = 0;
          lastposition = 0;
          break;       
    }
    showMode(workingMode);
    if (workingMode == 10) {
      lcd.setCursor(6,1); lcd.cursor();
    }
    curFreq = startFreq;
    inBlink=0;
  }
  if (workingMode == 9 && freqChanged==1) {
    freqChanged = 0;
    if (chipInUse == 0) {
      DDS9850.setfreq(startFreq);
    } else DDS9833.setfreq(startFreq);   
  }
  if (workingMode % 2 == 1 || workingMode==10) return;  // these workingMode no need freq mark
  if (markMethod == 0) {
    if (markpoint > 0) {
      markpoint--;
      if (workingMode == 6) { 
      //  delay(50); 
      }
      else {
          delayMicroseconds(delayCount);
      }
      return;
    }
  }
  if (count >= MAXCOUNT) {
    count = 0;
    if (workingMode == 6) {
      lastFreq=curFreq;
      curFreq = pgm_read_word_near(audioFreqList+0);   //in audio sweep mode, look freq from list
    } else 
      curFreq = startFreq;
    voltage = 0;
    markpoint=0;
  } else {
    count++;
    if (count == mark1 || count == mark2 || count == mark3 || count == mark4 || count == mark5 ||
        count == mark6 || count == mark7) {
      markpoint = markMethod==0 ? MAXMARKPOINT:2 ;
      if (sweepSpeed == 0) markpoint = markpoint + markpoint;
/*      if (workingMode == 8) {
        //testing
        Serial.print("mark:"); Serial.println(count);
        Serial.print("curFreq:"); Serial.println(curFreq+stepFreq);
        Serial.print("stepFreq:"); Serial.println(stepFreq);
   //     delay(1000);
      }*/
    }
    if (workingMode == 6) {
      lastFreq=curFreq;
      curFreq = pgm_read_word_near(audioFreqList+count);
//      Serial.print(" F:");Serial.print(curFreq);
    } else
      curFreq = curFreq + stepFreq;
    voltage += 4;
  }
  if (workingMode == 6){
    if (lastFreq != curFreq) {
      if (chipInUse == 0) {
            DDS9850.setfreq(curFreq);
          } else DDS9833.setfreq(curFreq); 
    }
  } else {
        if (chipInUse == 0) {
            DDS9850.setfreq(curFreq);
          } else DDS9833.setfreq(curFreq); 
  } 
  dac.analogWrite(voltage);
 
  if (markpoint > 0 && markMethod == 1) {
    if (markpoint > 1) 
      digitalWrite(A0,LOW);
    else 
      digitalWrite(A0,HIGH);
    markpoint--;   
  }
  if (workingMode == 6) { // Audio sweep
    delay(AUDIO_DELAY);     
    }
  else {
      delayMicroseconds(delayCount);
  }
}
